import React, { useState } from "react";
import { apiCall } from "../api/apiClient";

interface LoginProps {
  onLoginSuccess: (token: string) => void;
  onSwitchToRegister: () => void;
  isDarkMode: boolean;
}

export const Login: React.FC<LoginProps> = ({ 
  onLoginSuccess, 
  onSwitchToRegister, 
  isDarkMode 
}) => {
  const [username, setUsername] = useState("");
  const [password, setPassword] = useState("");
  const [error, setError] = useState<string | null>(null);
  const [loading, setLoading] = useState(false);

  async function handleSubmit(e: React.FormEvent) {
    e.preventDefault();
    setLoading(true);
    setError(null);

    try {
      const res = await apiCall("/api/login", {
        method: "POST",
        headers: { "Content-Type": "application/json" },
        body: JSON.stringify({ username, password }),
      });

      if (!res.ok) {
        const errorText = await res.text();
        throw new Error(`Login failed: ${res.status} ${res.statusText} - ${errorText}`);
      }

      const data = await res.json();
      onLoginSuccess(data.token);
    } catch (err: any) {
      setError(err.message || "Unknown error");
    } finally {
      setLoading(false);
    }
  }

  return (
    <div className={`gradient-bg ${isDarkMode ? 'gradient-bg-dark' : 'gradient-bg-light'}`}>
      <div
        className={`auth-container ${
          isDarkMode ? 'auth-container-dark' : 'auth-container-light'
        }`}
      >
        <h2 className={`auth-title ${isDarkMode ? 'text-white' : 'text-gray-500'}`}>
          Login - Litecloud
        </h2>

        {error && (
          <div className="error-message">
            {error}
          </div>
        )}

        <form onSubmit={handleSubmit} className="auth-form">
          <input
            type="text"
            placeholder="Username"
            value={username}
            onChange={(e) => setUsername(e.target.value)}
            className={`auth-input ${
              isDarkMode ? 'auth-input-dark' : 'auth-input-light'
            }`}
            required
          />
          
          <input
            type="password"
            placeholder="Password"
            value={password}
            onChange={(e) => setPassword(e.target.value)}
            className={`auth-input ${
              isDarkMode ? 'auth-input-dark' : 'auth-input-light'
            }`}
            required
          />

          <button
            type="submit"
            disabled={loading}
            className={`auth-button ${
              isDarkMode ? 'auth-button-dark' : 'auth-button-light'
            }`}
          >
            {loading ? "Logging in..." : "Login"}
          </button>
        </form>

        <button
          type="button"
          onClick={onSwitchToRegister}
          className={`auth-switch ${
            isDarkMode ? 'auth-switch-dark' : 'auth-switch-light'
          }`}
        >
          New user? Register now
        </button>
      </div>
    </div>
  );
};